@@ -10,7 +10,7 @@
  *--------------------------------------------------------------------------*/
 
 var GvaScript = {
-  Version: '1.09'
+  Version: '1.10'
 }
 
 //----------protoExtensions.js
@@ -59,7 +59,7 @@ Object.extend(Element, {
   },
 
 
-  autoScroll: function(elem, percentage) {
+  autoScroll: function(elem, container, percentage) {
      percentage = percentage || 20; // default                  
      var parent = elem.offsetParent;
      var offset = elem.offsetTop;
@@ -82,26 +82,30 @@ Object.extend(Element, {
        // .. TODO: need to investigate further how firefox handles offsets.
      }
 
-     var min = offset - (parent.clientHeight * (100-percentage)/100);
-     var max = offset - (parent.clientHeight * percentage/100);
-     if      (parent.scrollTop < min) parent.scrollTop = min;
-     else if (parent.scrollTop > max) parent.scrollTop = max;
+     container = container || parent;
+
+     var min = offset - (container.clientHeight * (100-percentage)/100);
+     var max = offset - (container.clientHeight * percentage/100);
+     if      (container.scrollTop < min) container.scrollTop = min;
+     else if (container.scrollTop > max) container.scrollTop = max;
   },
 
   outerHTML: function(elem) {
-     var tag = elem.tagName;
-     if (!tag)
-         return elem;
-     if (elem.outerHTML) {
-         return elem.outerHTML;
-     } else {
-         var attrs = elem.attributes;
-         var str = "<" + tag;
-         for (var i = 0; i < attrs.length; i++)
-             str += " " + attrs[i].name + "=\"" + attrs[i].value + "\"";
-
-         return str + ">" + elem.innerHTML + "</" + elem.tagName + ">";
-     }
+    var tag = elem.tagName;
+    if (!tag)
+      return elem;           // not an element node
+    if (elem.outerHTML)
+      return elem.outerHTML; // has builtin implementation 
+    else {
+      var attrs = elem.attributes;
+      var str = "<" + tag;
+      for (var i = 0; i < attrs.length; i++) {
+        var val = attrs[i].value;
+        var delim = val.indexOf('"') > -1 ? "'" : '"';
+        str += " " + attrs[i].name + "=" + delim + val + delim;
+      }
+      return str + ">" + elem.innerHTML + "</" + tag + ">";
+    }
   }
 
 });
@@ -397,15 +401,17 @@ GvaScript.TreeNavigator = function(elem, options) {
   // default options
   var defaultOptions = {
     tabIndex            : -1,
+    treeTabIndex        :  0,
     flashDuration       : 200,     // milliseconds
     flashColor          : "red",
-    selectDelay         : 200,     // milliseconds
+    selectDelay         : 100,     // milliseconds
     selectOnButtonClick : false,
+    noPingOnFirstClick  : false,
+    selectFirstNode     : true,
     createButtons       : true,
     autoScrollPercentage: 20,
     classes             : {},
-    keymap              : null,
-    selectFirstNode     : true
+    keymap              : null
   };
 
   this.options = Class.checkOptions(defaultOptions, options);
@@ -463,6 +469,9 @@ GvaScript.TreeNavigator = function(elem, options) {
   var numHandler = this._chooseLevel.bindAsEventListener(this);
   $R(1, 9).each(function(num){keyHandlers["C_" + num] = numHandler});
 
+  // tabIndex for the tree element
+  elem.tabIndex = elem.tabIndex || this.options.treeTabIndex;
+
   if (options.keymap) {
     this.keymap = options.keymap;
     this.keymap.rules.push(keyHandlers);
@@ -470,16 +479,19 @@ GvaScript.TreeNavigator = function(elem, options) {
   else {
     this.keymap = new GvaScript.KeyMap(keyHandlers);
 
-    // if the tree labels have no tabIndex, only the document receives 
-    // keyboard events
-    var target = this.options.tabIndex < 0 ? document : elem;
-    this.keymap.observe("keydown", target, {preventDefault:false,
-                                            stopPropagation:false});
+    // observe keyboard events on tree (preferred) or on document
+    var target = (elem.tabIndex  < 0) ? document : elem;
+    this.keymap.observe("keydown", target, Event.stopNone);
   }
 
   // selecting the first node
-  if (this.options.selectFirstNode)
+  if (this.options.selectFirstNode) {
     this.select(this.firstSubNode());
+
+    // if labels do not take focus but tree does, then set focus on the tree
+    if (this.options.tabIndex < 0 && elem.tabIndex >= 0)
+      elem.focus();
+  }
 }
 
 
@@ -574,6 +586,8 @@ GvaScript.TreeNavigator.prototype = {
       if (!content) {
         content = document.createElement('div');
         content.className = this.classes.content;
+        var content_type = node.getAttribute('content_type');
+        if (content_type) content.className += " " + content_type;
         content.innerHTML = "loading " + url;
         node.insertBefore(content, null); // null ==> insert at end of node
       }
@@ -590,7 +604,7 @@ GvaScript.TreeNavigator.prototype = {
   },
 
   select: function (node) {
-    var previousNode  = this.selectedNode;
+    var previousNode = this.selectedNode;
 
     // re-selecting the current node is a no-op
     if (node == previousNode) return;
@@ -601,31 +615,35 @@ GvaScript.TreeNavigator.prototype = {
         if (label) Element.removeClassName(label, this.classes.selected);
     }
 
-    // register code to call the selection handlers after some delay
-    var now = (new Date()).getTime(); 
-    this._lastSelectTime = now;
-    if (! this._selectionTimeoutId) {
-      var callback = this._selectionTimeoutHandler.bind(this, previousNode);
-      this._selectionTimeoutId = 
-        setTimeout(callback, this.options.selectDelay);
-    }
 
     // select the new node
+    var now = (new Date()).getTime(); 
+    if (node)
+        this._lastSelectTime = now;
     this.selectedNode = node;
     if (node) {
       this._assertNodeOrLeaf(node, 'select node');
       var label = this.label(node);
-      if (label) {
+      if (!label) {
+        throw new Error("selected node has no label");
+      }
+      else {
         Element.addClassName(label, this.classes.selected);
 
         if (this.isVisible(label)) {
           if (this.options.autoScrollPercentage !== null)
-            Element.autoScroll(label, this.options.autoScrollPercentage);
-          if (this.options.tabIndex >= 0)          
-            label.focus();
+            Element.autoScroll(label, 
+                               this.rootElement, 
+                               this.options.autoScrollPercentage);
         }
       }
-      else throw new Error("selected node has no label");
+    }
+
+    // register code to call the selection handlers after some delay
+    if (! this._selectionTimeoutId) {
+      var callback = this._selectionTimeoutHandler.bind(this, previousNode);
+      this._selectionTimeoutId = 
+        setTimeout(callback, this.options.selectDelay);
     }
   },
 
@@ -797,12 +815,48 @@ GvaScript.TreeNavigator.prototype = {
   },
   
   _labelClickHandler : function(event, label) {
-    var node = label.parentNode;
-    this.select(node);
-    var to_stop = this.fireEvent("Ping", node, this.rootElement);
-    Event.detailedStop(event, to_stop || Event.stopAll);
+    var node  = Element.navigateDom(label, 'parentNode',
+                                    this.classes.nodeOrLeaf);
+
+    // situation before the click
+    var was_selected = this.selectedNode == node;
+    var now = (new Date()).getTime(); 
+    var just_selected = (now - this._lastSelectTime < this.options.selectDelay);
+ 
+    // select node if necessary
+    if (!was_selected) this.select(node);
+
+    // should ping : depends on options.noPingOnFirstClick
+    var should_ping = (was_selected && !just_selected) 
+                    || !this.options.noPingOnFirstClick;
+
+    // do the ping if necessary
+    var event_stop_mode;
+    if (should_ping)
+      event_stop_mode = this.fireEvent("Ping", node, this.rootElement);
+
+    // avoid a second ping from the dblclick handler
+    this.should_ping_on_dblclick = !should_ping; 
+
+    // stop the event unless the ping_handler decided otherwise
+    Event.detailedStop(event, event_stop_mode || Event.stopAll);
   },
 
+
+  _labelDblClickHandler : function(event, label) {
+    var event_stop_mode;
+
+    // should_ping_on_dblclick was just set within _labelClickHandler
+    if (this.should_ping_on_dblclick) {
+      var node = label.parentNode;
+      event_stop_mode = this.fireEvent("Ping", node, this.rootElement);
+    }
+
+    // stop the event unless the ping_handler decided otherwise
+    Event.detailedStop(event, event_stop_mode || Event.stopAll);
+  },
+
+
   _buttonClickHandler : function(event) {
     var node = Event.element(event).parentNode;
     var method = this.isClosed(node) ? this.open : this.close;
@@ -824,6 +878,9 @@ GvaScript.TreeNavigator.prototype = {
       Event.observe(
         label,  "click",
         this._labelClickHandler.bindAsEventListener(this, label));
+      Event.observe(
+        label,  "dblclick",
+        this._labelDblClickHandler.bindAsEventListener(this, label));
       if (this.options.createButtons) {
         var button = document.createElement("span");
         button.className = this.classes.button;
@@ -837,17 +894,44 @@ GvaScript.TreeNavigator.prototype = {
 
   _addTabbingBehaviour: function(labels) {
     if (this.options.tabIndex < 0) return; // no tabbing
-    var treeNavigator = this;
+
+    // focus and blur do not bubble, so we'll have to insert them
+    // in each label element
+
+    var treeNavigator = this; // handlers will be closures on this
+
+    // focus handler
     var focus_handler = function(event) {
       var label = Event.element(event);
+      label.setAttribute('hasFocus', true);
+
       var node  = Element.navigateDom(label, 'parentNode',
                                       treeNavigator.classes.nodeOrLeaf);
                                                  
-      if (node) treeNavigator.select(node); 
+      // Select, but only if focus was not the consequence of a select action!
+      // To distinguish, we use the timestamp of the last select.
+      var now = (new Date()).getTime(); 
+      var short_delay = 2 * treeNavigator.options.selectDelay;
+         // needed to multiply by 2 because focus() is called indirectly by 
+         // _selectionTimeoutHandler after selectDelay milliseconds
+      if (node && now - treeNavigator._lastSelectTime > short_delay)
+        treeNavigator.select(node); 
     };
+
+    // blur handler
     var blur_handler = function(event) {
-      treeNavigator.select(null);
+      var label = Event.element(event);
+      label.setAttribute('hasFocus', false);
+
+      // Deselect, but only if blur was not the consequence of a select action!
+      // To distinguish, we use the timestamp of the last select.
+      var now = (new Date()).getTime(); 
+      var short_delay = 2 * treeNavigator.options.selectDelay;
+      if (now - treeNavigator._lastSelectTime > short_delay)
+        treeNavigator.select(null);
     };
+
+    // apply to each label
     labels.each(function(label) {
                   label.tabIndex = treeNavigator.options.tabIndex;
                   Event.observe(label, "focus", focus_handler);
@@ -874,9 +958,23 @@ GvaScript.TreeNavigator.prototype = {
       this._selectionTimeoutId = 
         setTimeout(callback, deltaDelay + 100); // allow for 100 more milliseconds
     }
-    else { // do the real work
+
+    // else do the real work
+    else { 
       this._selectionTimeoutId = null;
       var newNode = this.selectedNode;
+
+      // set focus
+      if (newNode) {
+        var label = this.label(newNode);
+        if (label && this.options.tabIndex >= 0 
+                  && !label.getAttribute('hasFocus') 
+                  && this.isVisible(label)) {
+            label.focus();
+        }
+      }
+
+      // fire events
       if (previousNode != newNode) {
         if (previousNode) 
           this.fireEvent("Deselect", previousNode, this.rootElement);
@@ -895,11 +993,11 @@ GvaScript.TreeNavigator.prototype = {
     var selectedNode = this.selectedNode;
     if (selectedNode) {
       var nextNode = this.nextDisplayedNode(selectedNode);
-      if (nextNode)
+      if (nextNode) {
         this.select(nextNode);
-      else
-        this.flash(selectedNode); 
-      Event.stop(event);
+        Event.stop(event);
+      }
+      // otherwise: do nothing and let default behaviour happen
     }
   },
 
@@ -907,11 +1005,11 @@ GvaScript.TreeNavigator.prototype = {
     var selectedNode = this.selectedNode;
     if (selectedNode) {
       var prevNode = this.previousDisplayedNode(selectedNode);
-      if (prevNode)
+      if (prevNode) {
         this.select(prevNode);
-      else
-        this.flash(selectedNode); 
-      Event.stop(event);
+        Event.stop(event);
+      }
+      // otherwise: do nothing and let default behaviour happen
     }
   },
 
@@ -1065,7 +1163,9 @@ GvaScript.ChoiceList = function(choices, options) {
     idForChoices     : "CL_choice",
     keymap           : null,
     grabfocus        : false,
-    scrollCount      : 5
+    scrollCount      : 5,
+    choiceItemTagName: "div",
+    htmlWrapper      : function(html) {return html;}
   };
 
 
@@ -1153,11 +1253,12 @@ GvaScript.ChoiceList.prototype = {
       id += this.options.idForChoices + "." + i;
       html += this.choiceElementHTML(label, id);
     }
-    return html;
-  }, 
+    return this.options.htmlWrapper(html);
+  },
 
   choiceElementHTML: function(label, id) {
-    return "<div class='" + this.classes.choiceItem +  "' id='" + id + "'>" + label + "</div>";
+    return "<" + this.options.choiceItemTagName + " class='" + this.classes.choiceItem +  "' id='" + id + "'>" 
+           + label + "</" + this.options.choiceItemTagName + ">";
   },
 
   fireEvent: GvaScript.fireEvent, // must be copied here for binding "this" 
@@ -1194,7 +1295,7 @@ GvaScript.ChoiceList.prototype = {
     Element.addClassName(elem, this.classes.choiceHighlight);
 
     if (autoScroll) 
-      Element.autoScroll(elem, 30); // 30%
+      Element.autoScroll(elem, this.container, 30); // 30%
 
     this.fireEvent({type: "Highlight", index: newIndex}, elem, this.container);
   },
@@ -1300,10 +1401,18 @@ GvaScript.AutoCompleter = function(datasource, options) {
     minWidth         : 200,       // pixels
     offsetX          : 0,         // pixels
     strict           : false,     // will not force to take value from choices
+    completeOnTab    : true,     // will not force to take value from choices
     blankOK          : true,
     colorIllegal     : "red",
     scrollCount      : 5,
-    actionItems      : null       // choice items to invoke javascript method
+    actionItems      : null,       // choice items to invoke javascript method
+    multivalued      : false,
+    multivalue_separator :  /[;,\s\t]/,
+    choiceItemTagName: "div",
+    htmlWrapper      : function(html) {return html;},
+    observed_scroll  : null,      // observe the scroll of a given element and move the dropdown accordingly (useful in case of scrolling windows)
+    additional_params: null,        //additional parameters with optional default values (only in the case where the datasource is a URL)
+    http_method      : 'get' // when additional_params is set, we might to just pass them in the body of the request
   };
 
   // more options for array datasources
@@ -1315,12 +1424,19 @@ GvaScript.AutoCompleter = function(datasource, options) {
   this.options = Class.checkOptions(defaultOptions, options);
 
   var defaultClasses = {
+    loading         : "AC_loading",
     dropdown        : "AC_dropdown",
     message         : "AC_message",
-    loading         : "AC_loading",
-    action          : "AC_action"
+    action          : "AC_action"  // undocumented on purpose !
   };
   this.classes = Class.checkOptions(defaultClasses, this.options.classes);
+  
+  this.separator = new RegExp(this.options.multivalue_separator);
+  this.default_separator_char = " "; //character used when the values are joined
+
+  if (this.options.multivalued && this.options.strict) {
+    throw new Error("not allowed to have a multivalued autocompleter in strict mode");
+  }
 
   this.dropdownDiv = null;
 
@@ -1339,7 +1455,8 @@ GvaScript.AutoCompleter = function(datasource, options) {
 
   // prepare some stuff to be reused when binding to inputElements
   this.reuse = {
-    onblur : this._blurHandler.bindAsEventListener(this)
+    onblur  : this._blurHandler.bindAsEventListener(this),
+    onclick : this._clickHandler.bindAsEventListener(this)
   };
 }
 
@@ -1366,6 +1483,7 @@ GvaScript.AutoCompleter.prototype = {
       this.keymap.observe("keydown", elem, { preventDefault:false,
                                              stopPropagation:false});
       Element.observe(elem, "blur", this.reuse.onblur);
+      Element.observe(elem, "click", this.reuse.onclick);
     }
 
     // initialize time stamps
@@ -1394,6 +1512,24 @@ GvaScript.AutoCompleter.prototype = {
     Element.addClassName(div, this.classes.message);
   },
 
+  // set additional params for autocompleters that have more than 1 param;
+  // second param is the HTTP method (post or get)
+  setAdditionalParams : function(params, method) {
+    this.additional_params = params;           
+    this.http_method = method;
+  },
+
+  addAdditionalParam : function(param, value) {
+    this.additional_params[param] = value;
+  },
+
+  setHttpMethod : function(method) {
+    if (method != 'get' || method != 'post') {
+        alert('Unrecognised type of http method')
+    }
+    this.http_method = method;
+  }, 
+
   //
   // TODO: TO BE REMOVED OR COMMITED TO ALIEN PACKAGE
   //
@@ -1403,6 +1539,31 @@ GvaScript.AutoCompleter.prototype = {
 
   fireEvent: GvaScript.fireEvent, // must be copied here for binding "this" 
 
+  // Set the element for the AC to look at to adapt its position. If elem is
+  // null, stop observing the scroll.
+  set_observed_scroll : function(elem) {
+    if (!elem) {
+        Event.stopObserving(this.observed_scroll, 'scroll', correct_dropdown_position);
+        return;
+    }
+
+    this.observed_scroll = elem;
+    this.currentScrollTop = elem.scrollTop;
+    this.currentScrollLeft = elem.scrollLeft;
+    var correct_dropdown_position = function() {
+      if (this.dropdownDiv) {
+        var dim = Element.getDimensions(this.inputElement);
+        var pos = this.dropdownDiv.positionedOffset()
+        this.dropdownDiv.style.top  = pos.top - (this.observed_scroll.scrollTop - this.currentScrollTop) + "px";
+        this.dropdownDiv.style.left = pos.left - this.observed_scroll.scrollLeft + "px"; 
+      }
+      this.currentScrollTop = this.observed_scroll.scrollTop;
+      this.currentScrollLeft = this.observed_scroll.scrollLeft;
+    }
+
+    Event.observe(elem, 'scroll', correct_dropdown_position.bindAsEventListener(this));
+  },
+
 
 //----------------------------------------------------------------------
 // PRIVATE METHODS
@@ -1417,12 +1578,32 @@ GvaScript.AutoCompleter.prototype = {
         if (this._runningAjax)
           this._runningAjax.transport.abort();
         Element.addClassName(autocompleter.inputElement, this.classes.loading);
+        var toCompleteVal = this._getValueToComplete();
+        
+        //integrate possible additional parameters in the URL request
+        var additional_params = this.additional_params;// for example {C_ETAT_AVOC : 'AC'}
+        var http_method = this.http_method ? this.http_method : this.options.http_method;
+        var partial_url = '';
+        if (additional_params && http_method == 'get') {
+            for (var key in additional_params) {
+                partial_url += "&" + key + "=" + additional_params[key]; 
+            }
+        } 
+
+        var complete_url = datasource + toCompleteVal + partial_url;
         this._runningAjax = new Ajax.Request(
-          datasource + autocompleter.inputElement.value,
+          complete_url,
           {asynchronous: true,
+           method: http_method,
+           postBody: this.http_method == 'post' ? Object.toJSON(additional_params) : null,
+           contentType: "text/javascript",
            onSuccess: function(xhr) {
               autocompleter._runningAjax = null;
-              autocompleter.choices = eval("(" + xhr.responseText + ")");
+
+              // do nothing if aborted by the onblur handler
+              if(xhr.transport.status == 0) return;
+
+              autocompleter.choices = xhr.responseJSON;
               autocompleter._displayChoices();
            },
            onFailure: function(xhr) {
@@ -1440,7 +1621,7 @@ GvaScript.AutoCompleter.prototype = {
     else if (typeof datasource == "function") { // callback
       return function() {
         this.inputElement.style.backgroundColor = ""; // remove colorIllegal
-        this.choices = datasource(this.inputElement.value);
+        this.choices = datasource(this._getValueToComplete());
         return false; // not asynchronous
       };
     }
@@ -1451,7 +1632,8 @@ GvaScript.AutoCompleter.prototype = {
         if (this.options.ignorePrefix)
           this.choices = datasource;
         else {
-          var regex = new RegExp("^" + this.inputElement.value,
+            var toCompleteVal = this._getValueToComplete();
+            var regex = new RegExp("^" + toCompleteVal,
                                  this.options.caseSensitive ? "" : "i");
           var matchPrefix = function(choice) {
 	    var value;
@@ -1479,8 +1661,14 @@ GvaScript.AutoCompleter.prototype = {
 
     // check if this is a "real" blur, or just a clik on dropdownDiv
     if (this.dropdownDiv) {
-      var x = Event.pointerX(window.event);
-      var y = Event.pointerY(window.event);
+      var targ;
+      var e = event;
+      if (e.target) targ = e.target;
+      else if (e.srcElement) targ = e.srcElement;
+      if (targ.nodeType && targ.nodeType == 3) // defeat Safari bug
+          targ = targ.parentNode;
+      var x = Event.pointerX(e) || Position.cumulativeOffset(targ)[0];
+      var y = Event.pointerY(e) || Position.cumulativeOffset(targ)[1];
       if (Position.within(this.dropdownDiv, x, y)) {
         // not a "real" blur ==> bring focus back to the input element
         this.inputElement.focus(); // will trigger again this.autocomplete()
@@ -1491,7 +1679,6 @@ GvaScript.AutoCompleter.prototype = {
       }
     }
     if (this.options.strict) {
-
       // initially : not OK unless options.blankOK and value is empty
       var valueOK = this.options.blankOK && this.inputElement.value == "";
 
@@ -1503,12 +1690,10 @@ GvaScript.AutoCompleter.prototype = {
       }
 
       // else update choices and then check
-      else { 
+      else {
         var async = this.updateChoices(); 
-
         // can only check if in synchronous mode
         if (!async) {
-
           // if got one single choice, take the canonic form of that one
           if (this.choices.length == 1) { 
             this.inputElement.value 
@@ -1544,13 +1729,25 @@ GvaScript.AutoCompleter.prototype = {
         this.inputElement.style.backgroundColor = this.options.colorIllegal;
       }
     }
-        
+
+    if(this._runningAjax) this._runningAjax.transport.abort();
     this.fireEvent("Leave", this.inputElement);
     this.inputElement = null;
   },
 
+  _clickHandler: function(event) {
+    var x = event.offsetX || event.layerX; // MSIE || FIREFOX
+    if (x > Element.getDimensions(this.inputElement).width - 20) {
+        if ( this.dropdownDiv )
+            this._removeDropdownDiv(event);
+        else
+            this._keyDownHandler(event);
+    }
+  },
+
   _keyDownHandler: function(event) { 
-    var valueLength = (this.inputElement.value || "").length; 
+    var value = this._getValueToComplete(); 
+    var valueLength = (value || "").length; 
     if (valueLength < this.options.minimumChars)
       this.displayMessage("liste de choix  partir de " 
                             + this.options.minimumChars + " caractres");
@@ -1593,7 +1790,11 @@ GvaScript.AutoCompleter.prototype = {
 
     // OK, we really have to check the value now
     this._timeLastCheck = now;
-    var value = this.inputElement.value;
+    var value = this.inputElement.value; //normal case
+    if (this.options.multivalued) { 
+        var vals = (this.inputElement.value).split(this.separator);
+        var value = vals[-1];
+    }
     if (value != this.lastValue) {
       this.lastValue = value;
       this.choices = null; // value changed, so invalidate previous choices
@@ -1607,6 +1808,32 @@ GvaScript.AutoCompleter.prototype = {
     }
   },
 
+  // return the value to be completed; added for multivalued autocompleters                  
+  _getValueToComplete : function() {
+    var toCompleteVal = this.inputElement.value;
+    if (this.options.multivalued) {
+        vals = (this.inputElement.value).split(this.separator);
+        toCompleteVal = vals[vals.length-1].replace(/\s+/, "");
+    }
+    return toCompleteVal;
+  },
+
+  // set the value of the field; used to set the new value of the field once the user 
+  // pings a choice item 
+  _setValue : function(value) {              
+    if (!this.options.multivalued) {
+        this.inputElement.value = value;
+    } else {
+        var vals = (this.inputElement.value).split(this.separator);
+        var result = (this.separator).exec(this.inputElement.value);
+        if (result) {
+            var user_sep = result[0];
+        }
+        vals[vals.length-1] = value;
+        this.inputElement.value = (vals).join(user_sep); 
+    }
+  
+  },
 
   _typeAhead : function () {
     var curLen     = this.lastValue.length;
@@ -1634,6 +1861,14 @@ GvaScript.AutoCompleter.prototype = {
   _mkDropdownDiv : function() {
     this._removeDropdownDiv();
 
+    // if observed element for scroll, reposition
+    var movedUpBy = 0;
+    var movedLeftBy = 0;
+    if (this.observed_scroll) {
+        movedUpBy = this.observed_scroll.scrollTop;
+        movedLeftBy = this.observed_scroll.scrollLeft;
+    }
+
     // create div
     var div    = document.createElement('div');
     div.className = this.classes.dropdown;
@@ -1641,11 +1876,11 @@ GvaScript.AutoCompleter.prototype = {
     // positioning
     var coords = Position.cumulativeOffset(this.inputElement);
     var dim    = Element.getDimensions(this.inputElement);
-    div.style.left      = (coords[0]+this.options.offsetX) + "px";
-    div.style.top       = coords[1] + dim.height + "px";
+    div.style.left      = coords[0] + this.options.offsetX - movedLeftBy + "px";
+    div.style.top       = coords[1] + dim.height -movedUpBy + "px";
     div.style.maxHeight = this.options.maxHeight + "px";
     div.style.minWidth  = this.options.minWidth + "px";
-    div.style.zIndex    = 32767; //Seems to be the highest valide value
+    div.style.zIndex    = 32767; //Seems to be the highest valid value
 
     // insert into DOM
     document.body.appendChild(div);
@@ -1664,8 +1899,9 @@ GvaScript.AutoCompleter.prototype = {
 
 
   _displayChoices: function() {
+    var toCompleteVal = this._getValueToComplete();
     if (!this.choices) {
-      var asynch = this.updateChoices();
+      var asynch = this.updateChoices(toCompleteVal);
       if (asynch) return; // updateChoices() is responsible for calling back
     }
 
@@ -1681,8 +1917,10 @@ GvaScript.AutoCompleter.prototype = {
     if (this.choices.length > 0) {
       var ac = this;
       var cl = this.choiceList = new GvaScript.ChoiceList(this.choices, {
-        labelField : this.options.labelField,
-        scrollCount: this.options.scrollCount
+        labelField        : this.options.labelField,
+        scrollCount       : this.options.scrollCount,
+        choiceItemTagName : this.options.choiceItemTagName,
+        htmlWrapper       : this.options.htmlWrapper
       });
 
 
@@ -1708,7 +1946,10 @@ GvaScript.AutoCompleter.prototype = {
       cl.fillContainer(this._mkDropdownDiv());
 
       // playing with the keymap: when tabbing, should behave like RETURN
+      var autocompleter = this;
       cl.keymap.rules[0].TAB = cl.keymap.rules[0].S_TAB = function(event) {
+        if (!autocompleter.options.completeOnTab)
+            return;
         var index = cl.currentHighlightedIndex;
         if (index != undefined) {
           var elem = cl._choiceElem(index);
@@ -1750,29 +1991,356 @@ GvaScript.AutoCompleter.prototype = {
   },
 
 
-
+  //triggered by the onPing event on the choicelist, i.e. when the user selects
+  //one of the choices in the list
   _completeFromChoiceElem: function(elem) {
+    // identify the selected line and handle it
     var num = parseInt(elem.id.match(/\.(\d+)$/)[1], 10);
-
     var choice = this.choices[num];
-    if (!choice) throw new Error("choice number is out of range : " + num);
+    if (!choice && choice!="" && choice!=0) 
+        throw new Error("choice number is out of range : " + num);
     var action = choice['action'];
     if (action) {
         this._removeDropdownDiv(); 
         eval(action);
         return;
     }
+
+    // add the value to the input element
     var value = this._valueFromChoice(num);
-    if (value) {
-      this.inputElement.value = this.lastValue = value;
-      this.inputElement.jsonValue = choice;
+    //if (value) {
+      this.lastValue = value;
+      this._setValue(value)
+//      this.inputElement.value = this.lastValue = value;
+      this.inputElement.jsonValue = choice; //never used elsewhere?!
       this._removeDropdownDiv();
-      this.inputElement.select();
+      if (!this.options.multivalued) {
+        this.inputElement.select();
+      } 
       this.fireEvent({type: "Complete", index: num}, elem, this.inputElement); 
-    } else {
-    }
+    //} else {
+    //}
     // else WHAT ??
     //    - might have other things to trigger (JS actions / hrefs)
   }
 
 }
+
+
+//----------repeat.js
+GvaScript.Repeat = {
+
+//-----------------------------------------------------
+// Public methods
+//-----------------------------------------------------
+
+  init: function(elem) {
+    this._init_repeat_elements(elem);
+  },
+
+  add: function(placeholder) {
+
+    // find the placeholder element
+    if (typeof placeholder == "string" && !placeholder.match(/.placeholder$/))
+        placeholder += ".placeholder";
+    placeholder = $(placeholder);
+
+    // repeat properties
+    var repeat = placeholder.repeat;
+
+    // won't add beyond max
+    if (repeat.count >= repeat.max)
+      return;
+
+    // increment the repetition block count and update path
+    var path_ini = repeat.path; // for restoring later; see end of block
+    repeat.ix    = ++repeat.count;
+    repeat.path  = repeat.path + "." + repeat.ix;
+
+    // regex substitutions to build html for the new repetition block (can't
+    // do it with Template.replace() because working in only one namespace)
+    var regex = new RegExp("#{" + repeat.name + "\\.(\\w+)}", "g");
+    var repl  = function ($0, $1){return repeat[$1] || ""};
+    var html  = repeat.template.replace(regex, repl);
+
+    // insert into the DOM
+    placeholder.insert({before:html});
+    var insertion_block = $(repeat.path);
+  
+    // repetition block gets an event
+    placeholder.fireEvent("Repeat", insertion_block);
+
+    // deal with nested repeated sections
+    this._init_repeat_elements(insertion_block, repeat.path);
+
+    // restore initial path, because unlike Perl, JS has no "local" :-( 
+    repeat.path = path_ini;
+  },
+
+
+//-----------------------------------------------------
+// Private methods
+//-----------------------------------------------------
+
+
+  _init_repeat_elements: function(elem, path) {
+    elem = $(elem);
+    if (elem) {
+      var elements = this._find_repeat_elements(elem);
+      for (var i = 0; i < elements.length; i++) {
+        this._init_repeat_element(elements[i], path);
+      }
+    }
+  },
+
+  _find_repeat_elements: function(elem) {
+    var result = [];
+
+    // navigate DOM, do not recurse under "repeat" nodes
+    for (var child = elem.firstChild; child; child = child.nextSibling) {
+      var has_repeat = child.nodeType == 1 && child.getAttribute('repeat');
+      result.push(has_repeat ? child : this._find_repeat_elements(child));
+    }
+    return result.flatten();
+  },
+
+  _init_repeat_element: function(element, path) {
+    element = $(element);
+    path = path || element.getAttribute('repeat-prefix');
+
+    // hash to hold all properties of the repeat element
+    var repeat = {};
+    repeat.name  = element.getAttribute('repeat');
+    repeat.min   = element.getAttribute('repeat-min') || 1,
+    repeat.max   = element.getAttribute('repeat-max') || 99,
+    repeat.count = 0;
+    repeat.path  = (path ? path + "." : "") + repeat.name;
+
+    // create a new element (placeholder for new insertion blocks)
+    var placeholder_tag = element.tagName.match(/^(TR|TD|TBODY|THEAD|TH)$/i) 
+                          ? element.tagName 
+                          : 'SPAN';
+    var placeholder     = document.createElement(placeholder_tag);
+    placeholder.id = repeat.path + ".placeholder";
+    placeholder.fireEvent = GvaScript.fireEvent;
+    element.parentNode.insertBefore(placeholder, element);
+
+    // take this elem out of the DOM and into a string ...
+    {
+      // a) force the id that will be needed in the template)
+      element.id = "#{" + repeat.name + ".path}";
+
+      // b) remove "repeat*" attributes (don't want them in the template)
+      for (var prop in element) {
+        if (prop.match(/^repeat/i))  element.removeAttribute(prop, 0);
+      }
+
+      // c) remove from DOM and keep it as a template string
+      // outerHTML would be simpler, but not supported by Gecko :-((
+      repeat.template = Element.outerHTML(element);
+      element.remove();
+
+//       var div = document.createElement("div");
+//       element = 
+//       div.appendChild(element);
+//       repeat.template = div.innerHTML;
+
+    }
+
+    // store all properties within the placeholder
+    placeholder.repeat = repeat;
+
+    // initial repetition blocks 
+    var n_start = element.getAttribute('repeat-start') || repeat.min;
+    for (var i = 1; i <= n_start; i++) {
+      this.add(placeholder);
+    }
+  }
+
+};
+
+//----------form.js
+/* 
+TODO
+
+  - submit
+
+     enctype attr
+     after_submit:
+        - replace document
+        - replace values
+        - relace element
+        - others ?
+   - submit attrs on buttons
+       - action / method / enctype / replace / target
+
+  - "onreceive" event (response after submit)
+
+  - autofocus
+
+  - check prototype.js serialize on multivalues
+
+*/
+
+GvaScript.Form = {
+
+  init: function(form, tree) {
+    GvaScript.Repeat.init(form);
+    if (tree)
+      this.fill_from_tree(form, "", tree);
+
+    // TODO : enctype, onsubmit, etc.
+
+  },
+
+
+  to_hash: function(form) {
+    return $(form).serialize({hash:true});
+  },
+
+
+  to_tree: function(form) {
+    return this.expand_hash(this.to_hash(form));
+  },
+
+
+  fill_from_tree : function(form, field_prefix, tree) {
+    form = $(form);
+    if (!form)
+      return;
+
+    for (var key in tree) {
+      if (!tree.hasOwnProperty(key)) 
+        continue;
+      var val = tree[key];
+      var new_prefix = field_prefix ? field_prefix+'.'+key : key;
+
+      switch (typeof(val)) {
+
+      case "boolean" :
+        val = val ? "true" : "";
+        // NO break here
+
+      case "number":
+      case "string":
+        var elem = form[new_prefix];
+        if (elem) 
+          this._fill_from_value(elem, val); 
+        break;
+
+      case "object":
+        if (val instanceof Array) 
+          this._fill_from_array(form, new_prefix, val);
+        else 
+          this.fill_from_tree(form, new_prefix, val);
+        break;
+
+      case "function":
+      case "undefined":
+        // do nothing
+      }
+    }
+  },
+
+
+  _fill_from_array: function(form, field_prefix, array) {
+    for (var i=1; i < array.length; i++) {
+      var new_prefix = field_prefix + "." + i;
+
+      // if form has a named element, fill it
+      var elem = form[new_prefix];
+      if (elem)
+        this._fill_from_value(elem, array[i]);
+
+      // otherwise try to walk down to a repetition block
+      else {
+
+        // try to find an existing repetition block
+        elem = $(new_prefix); // TODO : check: is elem in form ?
+
+        // no repetition block found, try to instanciate one
+        if (!elem) { 
+          var placeholder = $(field_prefix + ".placeholder");
+          if (placeholder && placeholder.repeat) {
+            while (placeholder.repeat.count < i
+                   && placeholder.repeat.count < placeholder.repeat.max) {
+              GvaScript.Repeat.add(placeholder);
+            }
+            elem = $(new_prefix);
+          }
+        }
+
+        // recurse to the repetition block
+        if (elem)
+          this.fill_from_tree(form, new_prefix, array[i]);
+      }           
+    }
+  },
+
+
+
+  _fill_from_value: function(elem, val) {
+    
+    if (!(val instanceof Array)) 
+      val = [val]; // force into an array
+
+    if (elem.length !== undefined) { // if elem is a collection
+      for (var i=0; i < elem.length; i++) {
+        this._fill_from_value(elem.item(i), val);
+      }
+    }
+    else { // if elem is a single node
+      switch (elem.type) {
+        case "checkbox" :
+        case "radio":
+          elem.checked = val.include(elem.value);
+          break;
+
+        case "text" :
+        case "textarea" :
+        case "hidden" :
+          elem.value = val.join(",");
+          break;
+
+        case "select-one" :
+        case "select-multiple" :
+          $A(elem.options).each(function(opt){
+            opt.selected = val.include(opt.value);
+          });
+          break;
+
+        default:
+          alert("unexpected elem type : " + elem.type);
+      } // end switch
+    } // end if
+  }, // end function
+
+
+  // javascript version of CGI::Expand::expand_hash
+  expand_hash: function(flat_hash) {
+    var tree = {};
+
+    // iterate on keys in the flat hash
+    for (var k in flat_hash) {
+      var parts = k.split(/\./);
+      var loop = {tree: tree, key: "root"};
+
+      // iterate on path parts within the key
+      for (var i = 0 ; i < parts.length; i++) {
+        var part = parts[i];
+
+        // if no subtree yet, build it (Array or Object)
+        if (!loop.tree[loop.key])
+          loop.tree[loop.key] = part.match(/^\d+$/) ? [] : {};
+
+        // walk down to subtree
+        loop = {tree: loop.tree[loop.key], key:part};
+      }
+      // store value in leaf
+      loop.tree[loop.key] = flat_hash[k];
+    }
+    return tree.root;
+  }
+
+
+};
